<?php
/**
 * This file is part of PDepend.
 *
 * PHP Version 5
 *
 * Copyright (c) 2008-2017 Manuel Pichler <mapi@pdepend.org>.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *
 *   * Neither the name of Manuel Pichler nor the names of his
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * @copyright 2008-2017 Manuel Pichler. All rights reserved.
 * @license http://www.opensource.org/licenses/bsd-license.php BSD License
 */

namespace PDepend\Source\AST;

/**
 * This is a special implementation of the node iterator that will translate
 * a list of given {@link \PDepend\Source\AST\ASTClassOrInterfaceReference} holders
 * into a list of unique {@link \PDepend\Source\AST\AbstractASTClassOrInterface}
 * instances.
 *
 * @copyright 2008-2017 Manuel Pichler. All rights reserved.
 * @license http://www.opensource.org/licenses/bsd-license.php BSD License
 *
 * @extends ASTArtifactList<\PDepend\Source\AST\AbstractASTClassOrInterface>
 */
class ASTClassOrInterfaceReferenceIterator extends ASTArtifactList
{
    /**
     * Constructs a new reference iterator instance.
     *
     * @param \PDepend\Source\AST\ASTClassOrInterfaceReference[] $references List of
     *        references to concrete type instances.
     */
    public function __construct(array $references)
    {
        parent::__construct($this->createClassesAndInterfaces($references));
    }

    /**
     * This method creates a set of {@link \PDepend\Source\AST\AbstractASTClassOrInterface}
     * objects from the given reference array.
     *
     * @param  \PDepend\Source\AST\ASTClassOrInterfaceReference[] $references
     * @return \PDepend\Source\AST\AbstractASTClassOrInterface[]
     */
    protected function createClassesAndInterfaces(array $references)
    {
        $classesAndInterfaces = array();

        foreach ($references as $reference) {
            $classOrInterface = $reference->getType();

            if (isset($classesAndInterfaces[$classOrInterface->getId()])) {
                continue;
            }
            $classesAndInterfaces[$classOrInterface->getId()] = $classOrInterface;
        }

        return $classesAndInterfaces;
    }
}
